@@ -20,62 +20,80 @@ NULL
 #' MultiAssayExperiment - An integrative multi-assay class for experiment data
 #'
 #' @description
-#' The \code{MultiAssayExperiment} class can be used to manage results of
+#' The `MultiAssayExperiment` class can be used to manage results of
 #' diverse assays on a collection of specimen. Currently,  the class can handle
 #' assays that are organized instances of
 #' \code{\linkS4class{SummarizedExperiment}},
-#' \code{\linkS4class{ExpressionSet}}, \code{matrix},
+#' \code{\linkS4class{ExpressionSet}}, `matrix`,
 #' \code{\link[RaggedExperiment:RaggedExperiment-class]{RaggedExperiment}}
-#' (inherits from \code{\linkS4class{GRangesList}}), and \code{RangedVcfStack}.
-#' Create new \code{MultiAssayExperiment} instances with the homonymous
+#' (inherits from \code{\linkS4class{GRangesList}}), and `RangedVcfStack`.
+#' Create new `MultiAssayExperiment` instances with the homonymous
 #' constructor, minimally with the argument \code{\link{ExperimentList}},
-#' potentially also with the arguments \code{colData} (see section below) and
+#' potentially also with the arguments `colData` (see section below) and
 #' \code{\link{sampleMap}}.
 #'
 #' @details
 #' The dots (\code{\ldots}) argument allows the user to specify additional
 #' arguments in several instances.
 #' \itemize{
-#' \item subsetting \strong{[}: additional arguments sent to
-#'     \link[GenomicRanges:findOverlaps-methods]{findOverlaps}.
-#' \item mergeReplicates: used to specify arguments for the \code{simplify}
-#'     functional argument
-#' \item assay: may contain withDimnames, which is forwarded to assays
-#' \item combining \strong{c}: compatible \code{MultiAssayExperiment} classes
-#'     passed on to the \linkS4class{ExperimentList} constructor, can be a
-#'     \code{list}, \linkS4class{List}, or a series of named arguments. See
-#'     the examples below.
+#'     \item subsetting \strong{[}: additional arguments sent to
+#'         \link[GenomicRanges:findOverlaps-methods]{findOverlaps}.
+#'     \item mergeReplicates: used to specify arguments for the \code{simplify}
+#'         functional argument
+#'     \item assay: may contain withDimnames, which is forwarded to assays
+#'     \item combining \strong{c}: compatible \code{MultiAssayExperiment} classes
+#'         passed on to the \code{\linkS4class{ExperimentList}} constructor,
+#'         can be a \code{list}, \code{\linkS4class{List}}, or a series of
+#'         named arguments. See the examples below.
 #' }
 #'
 #' @section colData:
-#' The \code{colData} slot is a collection of primary specimen data valid
+#' The `colData` slot is a collection of primary specimen data valid
 #' across all experiments. This slot is strictly of class
 #' \code{\linkS4class{DataFrame}} but arguments for the constructor function
-#' allow arguments to be of class \code{data.frame} and subsequently coerced.
+#' allow arguments to be of class `data.frame` and subsequently coerced.
 #'
 #' @section ExperimentList:
 #' The \code{\link{ExperimentList}} slot is designed to contain results from
 #' each experiment/assay. It contains a \linkS4class{SimpleList}.
 #'
 #' @section sampleMap:
-#' The \code{\link{sampleMap}} contains a \code{DataFrame} of translatable
-#' identifiers of samples and participants or biological units. Standard column
-#' names of the sampleMap are "assay", "primary", and "colname".
+#' The \code{\link{sampleMap}} contains a `DataFrame` of translatable
+#' identifiers of samples and participants or biological units. The standard
+#' column names of the `sampleMap` are "assay", "primary", and "colname".
+#' Note that the "assay" column is a factor corresponding to the names of each
+#' experiment in the `ExperimentList`. In the case where these names do
+#' not match between the `sampleMap` and the experiments, the documented
+#' experiments in the `sampleMap` take precedence and experiments are
+#' dropped by the harmonization procedure. The constructor function will
+#' generate a `sampleMap` in the case where it is not provided and this
+#' method may be a 'safer' alternative for creating the `MultiAssayExperiment`
+#' (so long as the rownames are identical in the `colData`, if provided).
+#' An empty `sampleMap` may produce empty experiments if the levels of the
+#' "assay" factor in the `sampleMap` do not match the names in the
+#' `ExperimentList`.
 #'
 #' @slot ExperimentList A \code{\link{ExperimentList}} class object for
 #' each assay dataset
-#' @slot colData A \code{DataFrame} of all clinical/specimen data available
-#' across experiments
-#' @slot sampleMap A \code{DataFrame} of translatable identifiers
-#' of samples and participants
-#' @slot metadata Additional data describing the
-#' \code{MultiAssayExperiment} object
-#' @slot drops A metadata \code{list} of dropped information
 #'
-#' @param object,x A \code{MultiAssayExperiment} object
+#' @slot colData A `DataFrame` of all clinical/specimen data available
+#' across experiments
+#'
+#' @slot sampleMap A `DataFrame` of translatable identifiers
+#' of samples and participants
+#'
+#' @slot metadata Additional data describing the
+#' `MultiAssayExperiment` object
+#'
+#' @slot drops A metadata `list` of dropped information
+#'
+#' @param object,x A `MultiAssayExperiment` object
+#'
 #' @param ... Additional arguments for supporting functions. See details.
 #'
-#' @return A \code{MultiAssayExperiment} object
+#' @return A `MultiAssayExperiment` object
+#'
+#' @md
 #'
 #' @examples
 #' example("MultiAssayExperiment")
@@ -98,9 +116,10 @@ NULL
 #' colData(compMAE)
 #'
 #' @exportClass MultiAssayExperiment
-#' @seealso
-#'     \link{MultiAssayExperiment-methods} for slot modifying methods
+#'
+#' @seealso \link{MultiAssayExperiment-methods} for slot modifying methods,
 #'     \href{https://github.com/waldronlab/MultiAssayExperiment/wiki/MultiAssayExperiment-API}{MultiAssayExperiment API}
+#'
 #' @include ExperimentList-class.R
 setClass(
     "MultiAssayExperiment",
@@ -171,7 +190,10 @@ setClass(
     }
 
     experiments <- mendoapply(function(x, idx) {
-        x[, colnames(x) %in% idx, drop=FALSE]
+        colmatch <- colnames(x) %in% idx
+        if (!all(colmatch))
+            x <- x[, colmatch, drop=FALSE]
+        x
     }, experiments[assay], experiments_columns[assay])
 
     if (length(harmony))
@@ -188,16 +210,49 @@ setClass(
 #' a \code{sampleMap} or a \code{colData} set is provided. Please see the
 #' MultiAssayExperiment API documentation for more information.
 #'
+#' @section colData:
+#' The `colData` input can be either `DataFrame` or `data.frame` with
+#' subsequent coercion to DataFrame. The rownames in the `colData` must match
+#' the colnames in the experiments if no sampleMap is provided.
+#'
+#' @section experiments:
+#' The `experiments` input can be of class \linkS4class{SimpleList} or `list`.
+#' This input becomes the \code{\link{ExperimentList}}. Each element of the
+#' input `list` or `List` must be named, rectangular with two dimensions, and
+#' have `dimnames`.
+#'
+#' @section sampleMap:
+#' The \code{\link{sampleMap}} can either be input as `DataFrame` or
+#' `data.frame` with eventual coercion to `DataFrame`. The `sampleMap` relates
+#' biological units and biological measurements within each assay. Each row in
+#' the `sampleMap` is a single such link. The standard column names of the
+#' `sampleMap` are "assay", "primary", and "colname".  Note that the "assay"
+#' column is a factor corresponding to the names of each experiment in the
+#' `ExperimentList`. In the case where these names do not match between the
+#' `sampleMap` and the experiments, the documented experiments in the
+#' `sampleMap` take precedence and experiments are dropped by the harmonization
+#' procedure. The constructor function will generate a `sampleMap` in the case
+#' where it is not provided and this method may be a 'safer' alternative for
+#' creating the `MultiAssayExperiment` (so long as the rownames are identical
+#' in the `colData`, if provided).  An empty `sampleMap` may produce empty
+#' experiments if the levels of the "assay" factor in the `sampleMap` do not
+#' match the names in the `ExperimentList`.
+#'
 #' @param experiments A \code{list} or \link{ExperimentList} of all
 #' combined experiments
+#'
 #' @param colData A \code{\linkS4class{DataFrame}} or \code{data.frame} of
 #' characteristics for all biological units
+#'
 #' @param sampleMap A \code{DataFrame} or \code{data.frame} of assay names,
 #' sample identifiers, and colname samples
+#'
 #' @param metadata An optional argument of "ANY" class (usually list) for
 #' content describing the experiments
+#'
 #' @param drops A \code{list} of unmatched information
 #' (included after subsetting)
+#'
 #' @return A \code{MultiAssayExperiment} object that can store
 #' experiment and phenotype data
 #'
@@ -206,43 +261,51 @@ setClass(
 #' @export MultiAssayExperiment
 #' @seealso \link{MultiAssayExperiment-class}
 MultiAssayExperiment <-
-    function(experiments = ExperimentList(), colData = S4Vectors::DataFrame(),
+    function(
+        experiments = ExperimentList(),
+        colData = S4Vectors::DataFrame(),
         sampleMap = S4Vectors::DataFrame(
             assay = factor(),
             primary = character(),
-            colname = character()),
-            metadata = list(),
-            drops = list())
+            colname = character()
+        ),
+        metadata = list(),
+        drops = list()
+    )
 {
-    if (missing(experiments))
-        experiments <- ExperimentList()
-    else
-        experiments <- ExperimentList(experiments)
-
-    if (missing(colData)){
-        allsamps <- unique(unlist(unname(colnames(experiments))))
-        colData <- S4Vectors::DataFrame(row.names = allsamps)
-    } else if (!is(colData, "DataFrame"))
-        colData <- S4Vectors::DataFrame(colData)
+    experiments <- as(experiments, "ExperimentList")
 
+    allsamps <- unique(unlist(unname(colnames(experiments))))
     if (missing(sampleMap)) {
+        if (missing(colData))
+            colData <- S4Vectors::DataFrame(row.names = allsamps)
         sampleMap <- .sampleMapFromData(colData, experiments)
     } else {
-        sampleMap <- S4Vectors::DataFrame(sampleMap)
-        if (!all(c("assay", "primary", "colname") %in% colnames(sampleMap)))
-            stop("'sampleMap' does not have required columns")
-        if (!is.factor(sampleMap[["assay"]]))
-            sampleMap[["assay"]] <- factor(sampleMap[["assay"]])
-        if (!is.character(sampleMap[["primary"]])) {
-            warning("sampleMap[['primary']] coerced to character()")
-            sampleMap[["primary"]] <- as.character(sampleMap[["primary"]])
-        }
-        if (!is.character(sampleMap[["colname"]])) {
-            warning("sampleMap[['colname']] coerced to character()")
-            sampleMap[["colname"]] <- as.character(sampleMap[["colname"]])
-        }
+        if (missing(colData))
+            colData <- S4Vectors::DataFrame(
+                row.names = unique(sampleMap[["primary"]])
+            )
     }
 
+    colData <- as(colData, "DataFrame")
+    sampleMap <- as(sampleMap, "DataFrame")
+
+    if (!all(c("assay", "primary", "colname") %in% colnames(sampleMap)))
+        stop("'sampleMap' does not have required columns")
+    if (!is.factor(sampleMap[["assay"]]))
+        sampleMap[["assay"]] <- as.factor(sampleMap[["assay"]])
+    if (!is.character(sampleMap[["primary"]])) {
+        warning("sampleMap[['primary']] coerced to character()")
+        sampleMap[["primary"]] <- as.character(sampleMap[["primary"]])
+    }
+    if (!is.character(sampleMap[["colname"]])) {
+        warning("sampleMap[['colname']] coerced to character()")
+        sampleMap[["colname"]] <- as.character(sampleMap[["colname"]])
+    }
+
+    if ((isEmpty(sampleMap) && !missing(sampleMap)) & !isEmpty(experiments))
+        warning("An empty 'sampleMap' may cause unexpected behavior")
+
     bliss <- .harmonize(experiments, colData, sampleMap)
 
     new("MultiAssayExperiment",
@@ -309,7 +372,7 @@ MultiAssayExperiment <-
         rownames(colData(object)),
         sampleMap(object)[["primary"]]
     ))) {
-        msg <- "All samples in the 'sampleMap' must be in the 'colData'"
+        msg <- "All 'sampleMap[[primary]]' must be in 'rownames(colData)'"
         errors <- c(errors, msg)
     }
     if (!is.factor(sampleMap(object)[["assay"]])) {
@@ -329,7 +392,7 @@ MultiAssayExperiment <-
     if (!logchecks)
         NULL
     else
-        "All sample identifiers in the assays must be unique"
+        "All colname identifiers in assays must be unique"
 }
 
 .validMultiAssayExperiment <- function(object) {
@@ -385,7 +448,7 @@ setMethod("show", "MultiAssayExperiment", function(object) {
         "\n *Format() - convert ", "into a long or wide DataFrame",
         "\n assays() - convert ", sprintf("%s", c_elist),
         " to a SimpleList of matrices",
-        "\n exportClass() - save all data to files\n",
+        "\n exportClass() - save data to flat files\n",
     sep = "")
 })
 
@@ -424,8 +487,11 @@ setMethod("show", "MultiAssayExperiment", function(object) {
 #' }
 #'
 #' @param object,x A \code{MultiAssayExperiment} object
+#'
 #' @param name A column in \code{colData}
+#'
 #' @param value See details.
+#'
 #' @param ... Argument not in use
 #'
 #' @return Accessors: Either a \code{sampleMap}, \code{ExperimentList}, or
@@ -467,7 +533,8 @@ setMethod("colData", "MultiAssayExperiment", function(x, ...) {
 #' @exportMethod metadata
 #' @rdname MultiAssayExperiment-methods
 setMethod("metadata", "MultiAssayExperiment", function(x)
-    getElement(x, "metadata"))
+    c(getElement(x, "metadata"), drops = getElement(x, "drops"))
+)
 
 ### - - - - - - - - - - - - - - - - - - - - - - - -
 ### Getters
@@ -527,44 +594,86 @@ setReplaceMethod("sampleMap", c("MultiAssayExperiment", "ANY"),
 setGeneric("experiments<-", function(object, value)
     standardGeneric("experiments<-"))
 
+setGeneric("drops<-", function(x, ..., value) standardGeneric("drops<-"))
+
 #' @exportMethod experiments<-
 #' @rdname MultiAssayExperiment-methods
 setReplaceMethod("experiments", c("MultiAssayExperiment", "ExperimentList"),
     function(object, value) {
-        rebliss <- .harmonize(value,
-            colData(object),
-            sampleMap(object))
+        if (!any(names(object) %in% names(value)) && !isEmpty(object)) {
+            drops(object) <-
+                list(experiments = setdiff(names(object), names(value)))
+            warning("'experiments' dropped; see 'metadata'", call. = FALSE)
+        }
+        o_cnames <- colnames(object)
+        v_cnames <- colnames(value)
+        if (identical(o_cnames, v_cnames))
+            BiocGenerics:::replaceSlots(
+                object = object,
+                ExperimentList = value,
+                check = FALSE
+            )
+        else {
+            rebliss <- .harmonize(value, colData(object), sampleMap(object))
 
-        BiocGenerics:::replaceSlots(object,
-            ExperimentList = rebliss[["experiments"]],
-            colData = rebliss[["colData"]],
-            sampleMap = rebliss[["sampleMap"]],
-            check = FALSE
-        )
+            BiocGenerics:::replaceSlots(
+                object = object,
+                ExperimentList = rebliss[["experiments"]],
+                colData = rebliss[["colData"]],
+                sampleMap = rebliss[["sampleMap"]],
+                check = FALSE
+            )
+        }
     }
 )
 
+#' @exportMethod experiments<-
+#' @rdname MultiAssayExperiment-methods
+setReplaceMethod("experiments", c("MultiAssayExperiment", "List"),
+    function(object, value) {
+        value <- as(value, "ExperimentList")
+        experiments(object) <- value
+        object
+    }
+)
+
+
 #' @exportMethod colData<-
 #' @rdname MultiAssayExperiment-methods
 setReplaceMethod("colData", c("MultiAssayExperiment", "DataFrame"),
     function(x, value) {
-        rebliss <- .harmonize(experiments(x),
-            value,
-            sampleMap(x))
+        x_rnames <- rownames(colData(x))
+        v_rnames <- rownames(value)
 
-        BiocGenerics:::replaceSlots(x,
-            ExperimentList = rebliss[["experiments"]],
-            colData = rebliss[["colData"]],
-            sampleMap = rebliss[["sampleMap"]],
-            check = FALSE
-        )
+        if (!any(v_rnames %in% x_rnames) && !isEmpty(value))
+            stop("'rownames(value)' have no match in 'rownames(colData)';\n  ",
+                "See '?renamePrimary' for renaming primary units")
+
+        if (identical(x_rnames, v_rnames))
+            BiocGenerics:::replaceSlots(
+                object = x,
+                colData = value,
+                check = FALSE
+            )
+        else {
+            rebliss <- .harmonize(experiments(x), value, sampleMap(x))
+
+            BiocGenerics:::replaceSlots(
+                object = x,
+                ExperimentList = rebliss[["experiments"]],
+                colData = rebliss[["colData"]],
+                sampleMap = rebliss[["sampleMap"]],
+                check = FALSE
+            )
+        }
     }
 )
 
 #' @rdname MultiAssayExperiment-methods
 setReplaceMethod("colData", c("MultiAssayExperiment", "ANY"),
     function(x, value) {
-        stopifnot(is.data.frame(value))
+        if (!is.data.frame(value))
+            stop("'colData' can be either 'data.frame' or 'DataFrame'")
         value <- as(value, "DataFrame")
         `colData<-`(x, value)
     }
@@ -579,10 +688,19 @@ setReplaceMethod("colData", c("MultiAssayExperiment", "ANY"),
 #' @exportMethod metadata<-
 #' @rdname MultiAssayExperiment-methods
 setReplaceMethod("metadata", c("MultiAssayExperiment", "ANY"),
-                 function(x, ..., value) {
-                     slot(x, "metadata") <- value
-                     return(x)
-                 })
+    function(x, ..., value) {
+        slot(x, "metadata") <- value
+        return(x)
+})
+
+setReplaceMethod("drops", c("MultiAssayExperiment", "ANY"),
+    function(x, ..., value) {
+        anydrops <- getElement(x, "drops")[["experiments"]]
+        if (length(anydrops))
+            value[["experiments"]] <- union(anydrops, value[["experiments"]])
+        slot(x, "drops") <- value
+        return(x)
+})
 
 #' @exportMethod $<-
 #' @rdname MultiAssayExperiment-methods
@@ -616,8 +734,54 @@ setReplaceMethod("names", c("MultiAssayExperiment", "ANY"),
         check = FALSE)
 })
 
+#' @exportMethod colnames<-
+#' @rdname MultiAssayExperiment-methods
+setReplaceMethod("colnames", c("MultiAssayExperiment", "List"),
+    function(x, value)
+{
+    if (!is(value, "CharacterList"))
+        stop("'value' must be a 'CharacterList' in 'colnames(x) <- value'")
+    if (length(value) != length(x))
+        stop("'colnames(x)' and 'value' not equal in length")
+
+    cnames <- colnames(x)
+    if (!identical(lengths(value), lengths(cnames)))
+        stop("'value' names and lengths should all be identical to 'names(x)'")
+
+    samplemap <- sampleMap(x)
+    splitmap <- mapToList(samplemap)
+    splitmap <- Map(function(x, y) {
+        x[["colname"]] <- y
+        x
+    }, x = splitmap, y = value)
+    exps <- experiments(x)
+    exps <- S4Vectors::mendoapply(function(x, y) {
+        colnames(x) <- y
+        x
+    }, x = exps, y = value)
+
+    BiocGenerics:::replaceSlots(
+        object = x,
+        ExperimentList = exps,
+        sampleMap = listToMap(splitmap)
+    )
+})
+
+#' @exportMethod colnames<-
+#' @rdname MultiAssayExperiment-methods
+setReplaceMethod("colnames", c("MultiAssayExperiment", "list"),
+    function(x, value)
+{
+    value <- as(value, "CharacterList")
+    colnames(x) <- value
+    x
+})
+
+
 #' @exportMethod updateObject
+#'
 #' @param verbose logical (default FALSE) whether to print extra messages
+#'
 #' @describeIn MultiAssayExperiment Update old serialized MultiAssayExperiment
 #' objects to new API
 setMethod("updateObject", "MultiAssayExperiment",
@@ -638,7 +802,8 @@ setMethod("updateObject", "MultiAssayExperiment",
                 .rearrangeMap(sampleMap(object))
             else sampleMap(object),
             metadata = metadata(object),
-            drops = object@drops)
+            drops = getElement(object, "drops")
+        )
     })
 
 .mergeColData <- function(inlist) {
@@ -670,6 +835,7 @@ setMethod("updateObject", "MultiAssayExperiment",
 }
 
 #' @rdname MultiAssayExperiment-class
+#'
 #' @name coerce-MultiAssayExperiment
 #'
 #' @aliases coerce,list,MultiAssayExperiment-method
@@ -684,6 +850,13 @@ setMethod("updateObject", "MultiAssayExperiment",
 #'
 #'     `as(x, "MultiAssayExperiment")`
 #'
+#'   Convert a `MultiAssayExperiment` to `MAF` class object using the
+#'   \link[methods]{as} function.
+#'
+#' In the following example, `x` is a \linkS4class{MultiAssayExperiment}:
+#'
+#'     `MultiAssayExperimentToMAF(x)`
+#'
 #' @md
 #'
 #' @exportMethod coerce
