@@ -23,6 +23,7 @@
 #' useful to simplify resulting outputs.
 #'
 #' @examples
+#'
 #' data(miniACC)
 #' upsetSamples(miniACC)
 #' upsetSamples(miniACC, nameFilter = function(x) substr(x, 1, 5))
@@ -37,8 +38,8 @@ upsetSamples <- function(MultiAssayExperiment,
     nsets = length(MultiAssayExperiment), nintersects = 24, order.by = "freq",
     nameFilter = force, check.names = FALSE, ... )
 {
-    if (!requireNamespace("UpSetR"))
-        stop("Please install the 'UpSetR' package to make venn diagrams")
+    if (!requireNamespace("UpSetR", quietly = TRUE))
+        stop("Please install the 'UpSetR' package to use 'upsetSamples()'")
     mae <- MultiAssayExperiment
     datf <- do.call(
         function(...) { data.frame(..., check.names = check.names) },
