@@ -23,7 +23,9 @@ metadata(miniACC)
 miniACC[c("MAPK14", "IGFBP2"), , ]
 
 ## ---- results='hide'----------------------------------------------------------
-miniACC[, miniACC$pathologic_stage == "stage iv", ]
+stg4 <- miniACC$pathologic_stage == "stage iv"
+# remove NA values from vector
+miniACC[, stg4 & !is.na(stg4), ]
 
 ## ---- results='hide'----------------------------------------------------------
 miniACC[, , "RNASeq2GeneNorm"]
